/*
	    Textfile_unittest_target.

	     :
	-   ;
	-  ,      .

	      
	     "Hello, World!".
*/

#include <iostream>
#include <fstream>
#include <sstream>
#include <stdexcept>

void
show_usage()
	{
		std::cout << "test <out_file_name> <line_count>" << std::endl;
	}

void
parse_args(
	int argc, char ** argv,
	std::string & out_file_name,
	unsigned int & line_count )
	{
		if( 3 == argc )
			{
				out_file_name = argv[ 1 ];

				try
					{
						std::istringstream line_count_stream( argv[ 2 ] );
						line_count_stream.exceptions(
							std::ios::badbit | std::ios::failbit );
						line_count_stream >> line_count;
					}
				catch( const std::ios::failure & )
					{
						throw std::domain_error(
							"Unable to parse line_count argument: " +
							std::string( argv[ 2 ] ) );
					}
			}
		else
			{
				show_usage();
				throw std::runtime_error( "Invalid arguments" );
			}
	}

void
create_out_file(
	const std::string & file_name,
	unsigned int line_count )
	{
		std::ofstream to( file_name.c_str() );
		if( !to )
			throw std::runtime_error( "Unable to create file: " +
				file_name );

		while( line_count-- )
			to << "Hello, World!" << std::endl;
	}

int
main( int argc, char ** argv )
	{
		int rc = 1;

		try
			{
				std::string out_file_name;
				unsigned int line_count;
				parse_args( argc, argv, out_file_name, line_count );

				create_out_file( out_file_name, line_count );

				rc = 0;
			}
		catch( const std::exception & x )
			{
				std::cerr << "Oops! " << x.what() << std::endl;
			}

		return rc;
	}
