require 'test/unit'

require 'mxx_ru/binary_target'

class TC_BinaryTargetLibMethods < Test::Unit::TestCase
  def make_static_lib( name )
    Mxx_ru::BinaryLibrary.new( name, Mxx_ru::BinaryLibrary::STATIC )
  end

  def make_shared_lib( name )
    Mxx_ru::BinaryLibrary.new( name, Mxx_ru::BinaryLibrary::SHARED )
  end

  def make_any_lib( name )
    Mxx_ru::BinaryLibrary.new( name, Mxx_ru::BinaryLibrary::ANY )
  end

  def test_add_libs
    target = Mxx_ru::BinaryTarget.new( 'test_add_libs' )

    target.lib( 'A' )
    target.lib_static( 'B' )
    target.lib_shared( 'C' )

    assert_equal(
        [ make_any_lib( 'A' ),
          make_static_lib( 'B' ),
          make_shared_lib( 'C' ) ],
        target.mxx_required_libs )
    assert_equal( [], target.mxx_required_lib_paths )
  end

  def test_add_libs_with_paths
    target = Mxx_ru::BinaryTarget.new( 'test_add_libs_with_paths' )

    target.lib( 'A', 'lib' )
    target.lib_static( 'B', '/usr/lib' )
    target.lib_shared( 'C', '/usr/local/lib' )

    assert_equal(
        [ make_any_lib( 'A' ),
          make_static_lib( 'B' ),
          make_shared_lib( 'C' ) ],
        target.mxx_required_libs )
    assert_equal(
        [ 'lib',
          '/usr/lib',
          '/usr/local/lib' ],
        target.mxx_required_lib_paths )
  end

  def test_lib_no_conflict
    target = Mxx_ru::BinaryTarget.new( 'test_lib_no_conflict' )
    
    target.lib( 'A' )
    target.lib_static( 'A' )
    target.lib( 'A' )

    assert_nothing_raised {
      target.mxx_required_libs
    }

    target.lib_shared( 'B' )
    target.lib( 'B' )

    assert_nothing_raised {
      target.mxx_required_libs
    }
  end

  def test_lib_conflict
    target = Mxx_ru::BinaryTarget.new( 'test_lib_conflict' )

    target.lib_static( 'C' )
    target.lib( 'C' )
    target.lib_shared( 'C' )
    target.lib( 'C' )

    assert_raise( Mxx_ru::BinaryLibraryTypeConflictEx ) {
      target.mxx_required_libs
    }
  end

  def test_lib_conflict_ex_data
    target = Mxx_ru::BinaryTarget.new( 'test_lib_conflict_ex_data' )

    target.lib( 'A' )
    target.lib_static( 'A' )
    target.lib( 'A' )

    target.lib_static( 'C' )
    target.lib( 'C' )
    target.lib_shared( 'C' )
    target.lib( 'C' )

    target.lib_shared( 'B' )
    target.lib( 'B' )

    exception_raised = false

    begin
      target.mxx_required_libs
    rescue Mxx_ru::BinaryLibraryTypeConflictEx => x
      exception_raised = true

      # Name of conflicted library must be in message.
      assert( x.message =~ /'C'/ )
    end

    assert_equal( true, exception_raised )
  end

end

