require 'test/unit'

require 'mxx_ru/generators/impl/cpp/generation'

class TC_GeneratorsCpp < Test::Unit::TestCase
  MODULE = MxxRu::Generators::Impl::Cpp

  def test_empty_args
    # All values expected to be nil.

    result = MODULE::Options.parse( [], 'dummy banner', {} )

    assert_nil( result.target_name )
    assert_nil( result.project_path )
    assert_nil( result.implib_path )
    assert_nil( result.output_file )
  end

  def test_target_name
    result = MODULE::Options.parse(
        %w{-t some_target_name},
        '',
        {} )

    assert_equal( 'some_target_name', result.target_name )
  end

  def test_target_name_and_project_path
    result1 = MODULE::Options.parse(
        %w{some/path --target some_target},
        '',
        {} )

    assert_equal( 'some/path', result1.project_path )
    assert_equal( 'some_target', result1.target_name )

    result2 = MODULE::Options.parse(
        %w{--target another_target another/path},
        '',
        {} )

    assert_equal( 'another/path', result2.project_path )
    assert_equal( 'another_target', result2.target_name )
  end

  def test_output_file
    result = MODULE::Options.parse(
        %w{some/path -t some_target -o out.rb},
        '',
        {} )

    assert_equal( 'some/path', result.project_path )
    assert_equal( 'some_target', result.target_name )
    assert_equal( 'out.rb', result.output_file )
  end

  def test_implib_path_disabled
    assert_raise( OptionParser::InvalidOption ) do
      result = MODULE::Options.parse(
          %w{--implib-path lib}, '', {} )
    end

    assert_raise( OptionParser::InvalidOption ) do
      result = MODULE::Options.parse(
          %w{--implib-path lib}, '', { :implib_path => false } )
    end
  end

  def test_implib_path_enabled
    result = nil
    assert_nothing_raised( OptionParser::InvalidOption ) do
      result = MODULE::Options.parse(
          %w{--implib-path lib}, '', { :implib_path => true } )

      assert_equal( 'lib', result.implib_path )
    end
  end
end

