require 'test/unit'

require 'mxx_ru/generators/impl/cpp/generation'

class TC_GeneratorsCppTemplateParams < Test::Unit::TestCase
  MODULE = MxxRu::Generators::Impl::Cpp until defined?( MODULE )

  def test_empty_options
    t1 = MODULE::TemplateParams.new( MODULE::LIB, MODULE::Options.new )
    assert_equal( 'lib_target', t1.setup_target_function )
    assert_equal( false, t1.has_or_require_implib )
    assert_equal( MODULE::TemplateParams::YOUR_TARGET_NAME, t1.target_name )

    t2 = MODULE::TemplateParams.new( MODULE::DLL, MODULE::Options.new )
    assert_equal( 'dll_target', t2.setup_target_function )
    assert_equal( true, t2.has_or_require_implib )
    assert_equal( MODULE::TemplateParams::YOUR_IMPLIB_PATH, t2.implib_path )
  end

  def test_target_name_in_options
    o = MODULE::Options.new
    o.target_name = 'my.target'
    o.project_path = 'some/path'
    t = MODULE::TemplateParams.new( MODULE::LIB, o )

    assert_equal( o.target_name, t.target_name )
  end

  def test_project_path_in_options
    o = MODULE::Options.new
    o.project_path = 'some/path\for/target'
    t = MODULE::TemplateParams.new( MODULE::LIB, o )

    assert_equal( 'some.path.for.target', t.target_name )
  end

  def test_implib_path_in_options
    o = MODULE::Options.new
    o.implib_path = 'some/path'
    t = MODULE::TemplateParams.new( MODULE::EXE, o )

    assert_equal( o.implib_path, t.implib_path )
  end
end

