require 'test/unit'
require 'fileutils'

require 'mxx_ru/cpp/obj_placements/custom_subdir'

class TC_CustomSubdirObjPlacement < Test::Unit::TestCase
  FINAL_PATH = 'bin'
  INTERMEDIATE_PATH = 'intermediate'

  def teardown
    FileUtils.rm_r( FINAL_PATH ) if File.directory?( FINAL_PATH )
    FileUtils.rm_r( INTERMEDIATE_PATH ) if File.directory?( INTERMEDIATE_PATH )
  end

  def test_final_path
    p = Mxx_ru::Cpp::CustomSubdirObjPlacement.new(
        FINAL_PATH, INTERMEDIATE_PATH )

    r = p.get_exe( '', nil, nil )
    check_name_and_path( FINAL_PATH + '/', r )
    r = p.get_exe( 'exe', nil, nil )
    check_name_and_path( FINAL_PATH + '/exe', r )

    r = p.get_dll( '', nil, nil )
    check_name_and_path( FINAL_PATH + '/', r )
    r = p.get_dll( 'dll', nil, nil )
    check_name_and_path( FINAL_PATH + '/dll', r )

    r = p.get_lib( '', nil, nil )
    check_name_and_path( FINAL_PATH + '/', r )
    r = p.get_lib( 'lib', nil, nil )
    check_name_and_path( FINAL_PATH + '/lib', r )
  end

  def test_intermediate_path
    p = Mxx_ru::Cpp::CustomSubdirObjPlacement.new(
        FINAL_PATH, INTERMEDIATE_PATH )

    r = p.get_obj( nil, nil, nil )
    check_name_and_path( INTERMEDIATE_PATH, r )

    r = p.get_obj( '', nil, nil )
    check_name_and_path( INTERMEDIATE_PATH, r )

    r = p.get_obj( '.', nil, nil )
    check_name_and_path( INTERMEDIATE_PATH, r )

    r = p.get_obj( 'prj_1/src', nil, nil )
    check_name_and_path( File.join( INTERMEDIATE_PATH, 'prj_1/src' ), r )
  end

private
  def check_name_and_path( expected, actual )
    assert_equal( expected, actual )
    assert( File.directory?( expected ) )
  end
end

