require 'test/unit'
require 'fileutils'

require 'mxx_ru/cpp/obj_placements/prj_aware_runtime_subdir'

class TC_PrjAwareRuntimeSubdirObjPlacement < Test::Unit::TestCase
  FINAL_PATH = 'bin'

  class DummyTarget
    attr_reader :prj_alias
    attr_reader :mxx_runtime_mode
    def initialize(a_alias, a_runtime)
      @prj_alias = a_alias
      @mxx_runtime_mode = a_runtime
    end
  end

  class DummyToolset
    def make_identification_string
      'dummy_toolset'
    end
  end

  def teardown
    FileUtils.rm_r( FINAL_PATH ) if File.directory?( FINAL_PATH )
  end

  def test_final_path_no_toolset_name
    target = DummyTarget.new( 'some/prj.rb', MxxRu::Cpp::RUNTIME_RELEASE )
    toolset = DummyToolset.new

    p = MxxRu::Cpp::PrjAwareRuntimeSubdirObjPlacement.new( FINAL_PATH )

    expected_part = '/release'

    r = p.get_exe( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
    r = p.get_exe( 'exe', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/exe', r )

    r = p.get_dll( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
    r = p.get_dll( 'dll', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/dll', r )

    r = p.get_lib( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
    r = p.get_lib( 'lib', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/lib', r )
  end

  def test_final_path_with_toolset_name
    target = DummyTarget.new( 'some/prj.rb', MxxRu::Cpp::RUNTIME_DEBUG )
    toolset = DummyToolset.new

    p = MxxRu::Cpp::PrjAwareRuntimeSubdirObjPlacement.new( FINAL_PATH,
          MxxRu::Cpp::PrjAwareRuntimeSubdirObjPlacement::USE_COMPILER_ID )

    expected_part = '/dummy_toolset/debug'

    r = p.get_exe( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
    r = p.get_exe( 'exe', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/exe', r )

    r = p.get_dll( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
    r = p.get_dll( 'dll', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/dll', r )

    r = p.get_lib( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
    r = p.get_lib( 'lib', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/lib', r )
  end

  def test_intermediate_path_no_toolset_name
    target = DummyTarget.new( 'some/prj.rb', MxxRu::Cpp::RUNTIME_RELEASE )
    toolset = DummyToolset.new

    p = MxxRu::Cpp::PrjAwareRuntimeSubdirObjPlacement.new( FINAL_PATH )

    expected_part = '/_objs/release/some_prj_rb'

=begin
    r = p.get_obj( '', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )

    r = p.get_obj( '.', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part, r )
=end

    r = p.get_obj( 'prj_1/src', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/prj_1_src', r )

    r = p.get_obj( 'some/very/long/path/to/src/file', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/some___file-3caf2536e0e7', r )
  end

  def test_intermediate_path_with_toolset_name
    target = DummyTarget.new( 'some/prj.rb', MxxRu::Cpp::RUNTIME_DEBUG )
    toolset = DummyToolset.new

    p = MxxRu::Cpp::PrjAwareRuntimeSubdirObjPlacement.new( FINAL_PATH,
          MxxRu::Cpp::PrjAwareRuntimeSubdirObjPlacement::USE_COMPILER_ID )

    expected_part = '/_objs/dummy_toolset/debug/some_prj_rb'

    r = p.get_obj( 'prj_1/src', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/prj_1_src', r )

    r = p.get_obj( 'some/very/long/path/to/src/file', toolset, target )
    check_name_and_path( FINAL_PATH + expected_part + '/some___file-3caf2536e0e7', r )
  end

private
  def check_name_and_path( expected, actual )
    assert_equal( expected, actual )
    assert( File.directory?( expected ) )
  end
end

