require 'test/unit'

require 'mxx_ru/cpp'

class TC_PluralFormMethods < Test::Unit::TestCase
  def test_required_prj
    test_value = [ 'test_required_prj_sub1', 'test_required_prj_sub2' ]

    test_value.each { |p| Mxx_ru::Cpp::composite_target( p ) {} }

    t = Mxx_ru::Cpp::composite_target( 'test_required_prj' ) {
        required_prjs test_value
      }

    assert_equal( test_value.size, t.mxx_required_prjs.size )
    t.mxx_required_prjs.each { |p|
      assert( test_value.member?( p.prj_alias ) )
    }
  end

  def test_c_sources
    test_files = [ 'hello.c', 'greeting.c' ]

    t = Mxx_ru::setup_target(
      Mxx_ru::Cpp::Exe_target.new( 'test_c_sources' ) {
        target 'test_c_sources'

        c_sources test_files
      },
      false )

    defined_sources = t.mxx_c_files.inject( [] ) { |r, f| r << File.basename( f.name ) }

    assert_equal( test_files, defined_sources )
  end

  def test_cpp_sources
    test_files = [ 'hello.cpp', 'greeting.cpp' ]

    path = 'some/test'
    t = Mxx_ru::setup_target(
      Mxx_ru::Cpp::Exe_target.new( "#{path}/test_cpp_sources" ) {
        target 'test_cpp_sources'

        cpp_sources test_files
      },
      false )

    expected_sources = test_files.inject( [] ) { |r, f| r << "#{path}/#{f}" }
    defined_sources = t.mxx_cpp_files.inject( [] ) { |r, f| r << f.name }

    assert_equal( expected_sources, defined_sources )
  end

  def test_cpp_sources_with_special_prefix
    path = 'some/test'
    test_files = [ "#{path}/hello.cpp", "#{path}/greeting.cpp" ]

    t = Mxx_ru::setup_target(
      Mxx_ru::Cpp::Exe_target.new( "#{path}/test_cpp_sources" ) {
        target 'test_cpp_sources'

        cpp_sources test_files
      },
      false )

    defined_sources = t.mxx_cpp_files.inject( [] ) { |r, f| r << f.name }

    assert_equal( test_files, defined_sources )
  end
end

