require 'test/unit'

require 'mxx_ru/cpp/qt'

class CustomQtGen < Mxx_ru::Cpp::QtGen
  public :names_from_ui
end

class TC_UicResultSubdir < Test::Unit::TestCase

  def test_in_cwd_no_uic_result_subdir
    target = create_target( 'no_uic_result_subdir.rb' )
    qt = create_qt_gen( target )

    ui_names = qt.names_from_ui( "dlg.ui", target )
    assert_equal( "dlg.hpp", ui_names.header )
    assert_equal( "dlg.cpp", ui_names.source )
  end

  def test_in_subdir_no_uic_result_subdir
    target = create_target( 'some/path/no_uic_result_subdir.rb' )
    qt = create_qt_gen( target )

    path = 'some/path'

    ui_names = qt.names_from_ui( "#{path}/dlg.ui", target )
    assert_equal( "#{path}/dlg.hpp", ui_names.header )
    assert_equal( "#{path}/dlg.cpp", ui_names.source )
  end

  def test_in_cwd_with_uic_result_subdir
    target = create_target( 'with_uic_result_subdir.rb' )
    qt = create_qt_gen( target )
    qt.uic_result_subdir = 'uic'

    ui_names = qt.names_from_ui( "dlg.ui", target )
    assert_equal( "./uic/dlg.hpp", ui_names.header )
    assert_equal( "./uic/dlg.cpp", ui_names.source )
  end

  def test_in_subdir_with_uic_result_subdir
    target = create_target( 'some/path/with_uic_result_subdir.rb' )
    qt = create_qt_gen( target )
    qt.uic_result_subdir = 'uic'

    path = 'some/path'

    ui_names = qt.names_from_ui( "#{path}/dlg.ui", target )
    assert_equal( "#{path}/uic/dlg.hpp", ui_names.header )
    assert_equal( "#{path}/uic/dlg.cpp", ui_names.source )
  end

  def test_target_include_path
    target = create_target( 'some/path/with_uic_result_subdir.rb' )
    qt = create_qt_gen( target )
    qt.uic_result_subdir = 'uic'
    qt.ui( 'dlg.ui' )
    qt.ui( 'mainwnd/mainwnd.ui' )

    assert( target.mxx_all_include_paths.include?( 'some/path/uic' ) )
    assert( target.mxx_all_include_paths.include?( 'some/path/mainwnd/uic' ) )
  end

private
  def create_target( prj_alias )
    Mxx_ru::Cpp::ExeTarget.new( prj_alias )
  end

  def create_qt_gen( target )
    qt = CustomQtGen.new( target )
    qt.hpp_ext = '.hpp'
    qt.cpp_ext = '.cpp'
    qt
  end
end

