require 'test/unit'

require 'mxx_ru/makestyle_generator'
require 'mxx_ru/util'

class TC_makestyle_generator < Test::Unit::TestCase
  def test_new
    g1 = Mxx_ru::Makestyle_generator.new(
      [ "a" ], [ "b" ], [ "c" ], [ "d" ] )

    assert( [ "a" ] == g1.target_files )
    assert( [ "b" ] == g1.dependencies )
    assert( [ "c" ] == g1.build_cmds )
    assert( [ "d" ] == g1.clean_cmds )

    g2 = Mxx_ru::Makestyle_generator.new(
      [ "a", "b" ], [ "c", "d" ], [ "e", "f" ] )
    assert_equal( [ "a", "b" ], g2.target_files )
    assert_equal( [ "c", "d" ], g2.dependencies )
    assert_equal( [ "e", "f" ], g2.build_cmds )
    assert_equal( [], g2.clean_cmds )

    g3 = Mxx_ru::Makestyle_generator.new(
      "a", "b", "c", "d" )

    assert( [ "a" ] == g3.target_files )
    assert( [ "b" ] == g3.dependencies )
    assert( [ "c" ] == g3.build_cmds )
    assert( [ "d" ] == g3.clean_cmds )

  end

  def test_build_simple
    g1 = Mxx_ru::Makestyle_generator.new(
      "a", "b", "echo hello > a" )

    Mxx_ru::Util.delete_file( "a" )
    File.new( "b", "w" ).close

    g1.build( "dummy" )

    assert( FileTest.exist?( "a" ) )
  end

  def test_build_no_real_rebuild

    Mxx_ru::Util.delete_file( "t" )
    Mxx_ru::Util.delete_file( "d" )

    File.new( "d", "w" ).close
    File.new( "t", "w" ).close

    g = Mxx_ru::Makestyle_generator.new(
      "t", "d", "echo has been rebuilt > t" )

    assert_equal( 0, FileTest.size( "t" ) )
  end

  def test_build_multiple
    Mxx_ru::Util.delete_file( "t3" )

    File.new( "d", "w" ).close

    g = Mxx_ru::Makestyle_generator.new(
      [ "t1", "t2", "t3" ], [ "d" ],
      [
        "echo t1 > t1",
        "echo t2 > t2"
      ] )
    g.build( "dummy" )

    assert( FileTest.size( "t1" ) > 0 )
    assert( FileTest.size( "t2" ) > 0 )
    assert( !FileTest.exist?( "t3" ) )

    File.truncate( "t1", 0 )
    File.truncate( "t2", 0 )

    g.build( "dummy" )

    assert( FileTest.size( "t1" ) > 0 )
    assert( FileTest.size( "t2" ) > 0 )
    assert( !FileTest.exist?( "t3" ) )

  end

  def test_clean_simple
    File.new( "d", "w" ).close

    g = Mxx_ru::Makestyle_generator.new(
      [ "t1", "t2", "t3" ], [ "d" ],
      [
        "echo t1 > t1",
        "echo t2 > t2",
        "echo t3 > t3"
      ] )
    g.build( "dummy" )

    assert( FileTest.size( "t1" ) > 0 )
    assert( FileTest.size( "t2" ) > 0 )
    assert( FileTest.size( "t3" ) > 0 )

    g.clean( "dummy" )

    assert( !FileTest.exist?( "t1" ) )
    assert( !FileTest.exist?( "t2" ) )
    assert( !FileTest.exist?( "t3" ) )
  end

  def teardown
    [ "a", "b", "c", "d", "e",
      "t", "t1", "t2", "t3" ].each do |f|
        Mxx_ru::Util.delete_file( f )
      end
  end
end

