require 'test/unit'
require 'fileutils'

require 'mxx_ru/cpp/toolset'

class TC_TagSearch < Test::Unit::TestCase
  class SpecialObject
    def each( &block )
      [ 'first', 'second', 'third' ].each &block
    end
  end

  def setup
    @ts = MxxRu::Cpp::Toolset.new( 'dummy' )
  end

  def test_no_tag
    assert_equal( '12345', @ts.tag( 'missing', '12345' ) )

    assert_raise( MxxRu::Cpp::Toolset::UnknownTagEx ) do
      @ts.tag( 'missing' )
    end
  end

  def test_no_tag_array
    assert_equal( '12345', @ts.tag( [ 'one', 'two', 'three' ], '12345' ) )

    assert_raise( MxxRu::Cpp::Toolset::UnknownTagEx ) do
      @ts.tag( [ 'one', 'two', 'three' ] )
    end
  end

  def test_exist
    tag = 'defined'
    value = 'value'

    @ts.setup_tag( tag, value )

    assert_equal( value, @ts.tag( tag, 'wrong-search-result' ) )
  end

  def test_exist_array
    tags = [ 'first', 'second', 'third' ]
    values = [ '1', '2', '3' ]

    tags.zip( values ).each do |pair|
      @ts.setup_tag( *pair )
    end

    assert_equal( values[ 0 ], @ts.tag( tags, 'wrong-search-result' ) )
    assert_equal( values[ 1 ],
        @ts.tag( [ tags[1], tags[0], tags[2] ], 'wrong-search-result' ) )
  end

  def test_invalid_key_type
    assert_raise( MxxRu::InvalidValueEx ) do
      @ts.tag( 1 )
    end
  end

  def test_special_object_with_each

    @ts.setup_tag( 'second', '2' )
    assert_equal( '2', @ts.tag( SpecialObject.new, '2222' ) )
  end

end

