require 'test/unit'

require_relative 'restore_default_manifest'

class TC_ActualManifest < Test::Unit::TestCase
  def initialize( test_method_name )
    super test_method_name
  end

  def setup
    @old_manifest = Mxx_ru::Cpp::Toolsets::Vc8::default_manifest
    # Default manifest should not be used.
    Mxx_ru::Cpp::Toolsets::Vc8::manifest nil
  end

  def teardown
    Mxx_ru::Cpp::Toolsets::Vc8::restore_default_manifest @old_manifest
  end

  def make_exe_target( prj_alias )
    Mxx_ru::Cpp::Exe_target.new( prj_alias ) {
      target 'sample'
    }
  end

  def make_dll_target( prj_alias )
    Mxx_ru::Cpp::Dll_target.new( prj_alias ) {
      target 'sample'
    }
  end

  def produce_manifest( target, hash )
    Mxx_ru::Cpp::Toolsets::Vc8::manifest( hash )
    Mxx_ru::Cpp::toolset.setup_mandatory_options( target )
    
    target.vc8_actual_manifest
  end

  def test_target_autogen
    target = make_exe_target( 'target_autogen.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => :to_default_file )

    assert_not_nil( m, 'actual manifest expected' )
    assert_equal( true, m.autogen? )
    assert_equal( false, m.mt? )
    assert_equal( './sample.exe.manifest', m.manifest_file )
    assert_nil( m.digital_resource_id )
    assert_equal( true, m.keep_manifest_file? )
  end

  def test_target_autogen_embed
    target = make_exe_target( 'target_autogen_embed.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => {} )

    assert_not_nil( m, 'actual manifest expected' )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( './sample.exe.manifest', m.manifest_file )
    assert_equal( 1, m.digital_resource_id )
    assert_equal( false, m.keep_manifest_file? )
  end

  def test_target_autogen_embed_keep_manifest
    target = make_exe_target(
      'target_autogen_embed_keep_manifest.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => { :keep_manifest_file => true } )

    assert_not_nil( m, 'actual manifest expected' )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( './sample.exe.manifest', m.manifest_file )
    assert_equal( 1, m.digital_resource_id )
    assert_equal( true, m.keep_manifest_file? )
  end

  def test_dlltarget_autogen_embed
    target = make_dll_target( 'dlltarget_autogen_embed.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => {} )

    assert_not_nil( m, 'actual manifest expected' )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( './sample.dll.manifest', m.manifest_file )
    assert_equal( 2, m.digital_resource_id )
    assert_equal( false, m.keep_manifest_file? )
  end

  def test_target_autogen_embed_process_manifest
    target = make_exe_target(
      'target_autogen_embed_process_manifest.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => { :resource_id => :process_manifest } )

    assert_not_nil( m )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( './sample.exe.manifest', m.manifest_file )
    assert_equal( 1, m.digital_resource_id )
    assert_equal( false, m.keep_manifest_file? )
  end

  def test_dlltarget_autogen_embed_process_manifest
    target = make_dll_target(
      'dlltarget_autogen_embed_process_manifest.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => { :resource_id => :isolationaware_manifest } )

    assert_not_nil( m )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( './sample.dll.manifest', m.manifest_file )
    assert_equal( 2, m.digital_resource_id )
    assert_equal( false, m.keep_manifest_file? )
  end

  def test_target_autogen_file_embed
    target = make_exe_target(
      'target_autogen_file_embed.rb' )

    m = produce_manifest( target,
      :target => target,
      :autogen => 'hello_world.exe.manifest',
      :mt => {} )
    
    assert_not_nil( m )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( 'hello_world.exe.manifest', m.manifest_file )
    assert_equal( 1, m.digital_resource_id )
    assert_equal( false, m.keep_manifest_file? )
  end

  def test_target_embed_file
    target = make_exe_target(
      'target_embed_file.rb' )

    m = produce_manifest( target,
      :target => target,
      :mt => { :manifest => 'hello_world.exe.manifest' } )
    
    assert_not_nil( m )
    assert_equal( false, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( 'hello_world.exe.manifest', m.manifest_file )
    assert_equal( 1, m.digital_resource_id )
    assert_equal( true, m.keep_manifest_file? )
  end

  def test_dlltarget_embed_file
    target = make_dll_target(
      'dlltarget_embed_file.rb' )

    m = produce_manifest( target,
      :target => target,
      :mt => { :manifest => 'hello_world.dll.manifest' } )
    
    assert_not_nil( m )
    assert_equal( false, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( 'hello_world.dll.manifest', m.manifest_file )
    assert_equal( 2, m.digital_resource_id )
    assert_equal( true, m.keep_manifest_file? )
  end

  def test_target_embed_file_isolationaware_nostaticimport
    target = make_dll_target(
      'target_embed_file_isolationawate_nostaticimport.rb' )

    m = produce_manifest( target,
      :target => target,
      :mt => {
        :manifest => 'hello_world.dll.manifest',
        :resource_id => :isolationaware_nostaticimport_manifest } )
    
    assert_not_nil( m )
    assert_equal( false, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( 'hello_world.dll.manifest', m.manifest_file )
    assert_equal( 3, m.digital_resource_id )
    assert_equal( true, m.keep_manifest_file? )
  end

  def test_autogen
    target = make_exe_target(
      'target_autogen.rb' )

    m = produce_manifest( target,
      :autogen => :to_default_file )
    
    assert_not_nil( m )
    assert_equal( true, m.autogen? )
    assert_equal( false, m.mt? )
    assert_equal( './sample.exe.manifest', m.manifest_file )
    assert_nil( m.digital_resource_id )
    assert_equal( true, m.keep_manifest_file? )
  end

  def test_autogen_embed
    target = make_exe_target(
      'target_autogen_embed.rb' )

    m = produce_manifest( target,
      :autogen => :to_default_file,
      :mt => {} )
    
    assert_not_nil( m )
    assert_equal( true, m.autogen? )
    assert_equal( true, m.mt? )
    assert_equal( './sample.exe.manifest', m.manifest_file )
    assert_equal( 1, m.digital_resource_id )
    assert_equal( false, m.keep_manifest_file? )
  end
end


