require 'test/unit'

require 'mxx_ru/util'
require 'mxx_ru/cpp'
require 'mxx_ru/cpp/toolsets/vc8'

class TC_AppendMtCommands < Test::Unit::TestCase

  def make_exe_target( prj_alias )
    Mxx_ru::Cpp::Exe_target.new( prj_alias ) {
      target 'sample'
    }
  end

  def make_dll_target( prj_alias )
    Mxx_ru::Cpp::Dll_target.new( prj_alias ) {
      target 'sample'
    }
  end

  def produce_manifest( target, hash )
    Mxx_ru::Cpp::Toolsets::Vc8::manifest( hash )
    Mxx_ru::Cpp::toolset.setup_mandatory_options( target )
    
    target.vc8_actual_manifest
  end

  def test_exe_autogen_to_default_file
    target = make_exe_target( 'exe_autogen' )

    produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => {} )

    r = Mxx_ru::Cpp::toolset.make_exe_command_lines(
      './sample.exe',
      Mxx_ru::Cpp::Exe_info.new( 'sample.exe', './' ),
      Mxx_ru::Cpp::Linker_lists.new(),
      target )

    assert( r.size > 2, "r.size (#{r.size}) should be greater than 2" )
    assert_equal( "del \"#{Mxx_ru::Util::native_pathname('./sample.exe.manifest')}\"", r[ -1 ] )
    assert_equal( 'mt /nologo /outputresource:"./sample.exe;#1" ' +
      '/manifest "./sample.exe.manifest"', r[ -2 ] )
  end

  def test_exe_autogen_to_default_file_keep_manifest
    target = make_exe_target( 'exe_autogen_keep_manifest' )

    produce_manifest( target,
      :target => target,
      :autogen => :to_default_file,
      :mt => { :keep_manifest_file => true } )

    r = Mxx_ru::Cpp::toolset.make_exe_command_lines(
      './sample.exe',
      Mxx_ru::Cpp::Exe_info.new( 'sample.exe', './' ),
      Mxx_ru::Cpp::Linker_lists.new(),
      target )

    assert( r.size > 1, "r.size (#{r.size}) should be greater than 1" )
    assert_equal( 'mt /nologo /outputresource:"./sample.exe;#1" ' +
      '/manifest "./sample.exe.manifest"', r[ -1 ] )
  end

  def test_exe_noautogen_keep_manifest
    target = make_exe_target( 'exe_noautogen_keep_manifest' )

    produce_manifest( target,
      :target => target,
      :mt => { :manifest => 'some.manifest',
        :keep_manifest_file => true } )

    r = Mxx_ru::Cpp::toolset.make_exe_command_lines(
      './sample.exe',
      Mxx_ru::Cpp::Exe_info.new( 'sample.exe', './' ),
      Mxx_ru::Cpp::Linker_lists.new(),
      target )

    assert( r.size > 1, "r.size (#{r.size}) should be greater than 1" )
    assert_equal( 'mt /nologo /outputresource:"./sample.exe;#1" ' +
      '/manifest "some.manifest"', r[ -1 ] )
  end

  def test_exe_noautogen_nokeep_manifest
    target = make_exe_target( 'exe_noautogen_keep_nomanifest' )

    produce_manifest( target,
      :target => target,
      :mt => { :manifest => 'some.manifest' } )

    r = Mxx_ru::Cpp::toolset.make_exe_command_lines(
      './sample.exe',
      Mxx_ru::Cpp::Exe_info.new( 'sample.exe', './' ),
      Mxx_ru::Cpp::Linker_lists.new(),
      target )

    assert( r.size > 1, "r.size (#{r.size}) should be greater than 1" )
    assert_equal( 'mt /nologo /outputresource:"./sample.exe;#1" ' +
      '/manifest "some.manifest"', r[ -1 ] )
  end
end

