require 'test/unit'

require_relative 'restore_default_manifest'

class TC_DefineManifest < Test::Unit::TestCase
  def initialize( test_method_name )
    super test_method_name
  end

  def setup
    @old_manifest = Mxx_ru::Cpp::Toolsets::Vc8::default_manifest
    # Default manifest should not be used.
    Mxx_ru::Cpp::Toolsets::Vc8::manifest nil
  end

  def teardown
    Mxx_ru::Cpp::Toolsets::Vc8::restore_default_manifest @old_manifest
  end

  def test_target_autogen
    target = Mxx_ru::Cpp::Exe_target.new( 'target_autogen.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :autogen => :to_default_file )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( false, m.mt )
    assert_equal( :default, m.manifest_file )
    assert_nil( m.resource_id )
  end

  def test_target_autogen_embed
    target = Mxx_ru::Cpp::Exe_target.new( 'target_autogen_embed.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :autogen => :to_default_file,
      :mt => {} )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( :default, m.manifest_file )
    assert_equal( :default, m.resource_id )
    assert_equal( false, m.keep_manifest_file )
  end

  def test_target_autogen_embed_keep_manifest
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_autogen_embed_keep_manifest.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :autogen => :to_default_file,
      :mt => { :keep_manifest_file => true } )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( :default, m.manifest_file )
    assert_equal( :default, m.resource_id )
    assert_equal( true, m.keep_manifest_file )
  end

  def test_target_autogen_embed_process_manifest
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_autogen_embed_process_manifest.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :autogen => :to_default_file,
      :mt => { :resource_id => :process_manifest } )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( :default, m.manifest_file )
    assert_equal( :process_manifest, m.resource_id )
  end

  def test_target_autogen_file_embed
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_autogen_file_embed.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :autogen => 'hello_world.exe.manifest',
      :mt => {} )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( 'hello_world.exe.manifest', m.manifest_file )
    assert_equal( :default, m.resource_id )
  end

  def test_target_embed_file
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_embed_file.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :mt => { :manifest => 'hello_world.exe.manifest' } )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( false, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( 'hello_world.exe.manifest', m.manifest_file )
    assert_equal( :default, m.resource_id )
  end

  def test_target_embed_file_isolationaware_nostaticimport
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_embed_file_isolationawate_nostaticimport.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :target => target,
      :mt => {
        :manifest => 'hello_world.exe.manifest',
        :resource_id => :isolationaware_nostaticimport_manifest } )
    
    m = target.vc8_source_manifest

    assert_not_nil( m )
    assert_equal( false, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( 'hello_world.exe.manifest', m.manifest_file )
    assert_equal( :isolationaware_nostaticimport_manifest, m.resource_id )
  end

  def test_autogen
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_autogen.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :autogen => :to_default_file )
    
    m = Mxx_ru::Cpp::Toolsets::Vc8::default_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( false, m.mt )
    assert_equal( :default, m.manifest_file )
    assert_nil( m.resource_id )
  end

  def test_autogen_embed
    target = Mxx_ru::Cpp::Exe_target.new(
      'target_autogen_embed.rb' )

    Mxx_ru::Cpp::Toolsets::Vc8::manifest(
      :autogen => :to_default_file,
      :mt => {} )
    
    m = Mxx_ru::Cpp::Toolsets::Vc8::default_manifest

    assert_not_nil( m )
    assert_equal( true, m.autogen )
    assert_equal( true, m.mt )
    assert_equal( :default, m.manifest_file )
    assert_equal( :default, m.resource_id )
  end

end

