require 'test/unit'

require 'mxx_ru/cpp'
require 'mxx_ru/cpp/toolsets/vc8'

class TC_DefaultManifest < Test::Unit::TestCase
  def test_invalid_resource_id
    assert_nothing_raised() do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest \
        :autogen => :to_default_file,
        :mt => {}
    end

    assert_nothing_raised() do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest :mt => {
        :manifest => 'some_file',
        :resource_id => :default }

      Mxx_ru::Cpp::Toolsets::Vc8::manifest :mt => {
        :manifest => 'some_file',
        :resource_id => :process_manifest }

      Mxx_ru::Cpp::Toolsets::Vc8::manifest \
        :autogen => :to_default_file,
        :mt => { :resource_id => :isolationaware_manifest }

      Mxx_ru::Cpp::Toolsets::Vc8::manifest \
        :autogen => 'some_file',
        :mt => { :resource_id => :isolationaware_nostaticimport_manifest }
    end

    assert_raise( Mxx_ru::Invalid_value_ex ) do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest :mt => {
        :manifest => 'some_file',
        :resource_id => :process_manifest_id }
    end
  end

  def test_missing_manifest_name
    assert_raise( Mxx_ru::Invalid_value_ex ) do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest :mt => {
        :resource_id => :process_manifest }
    end
  end

  def test_invalid_autogen
    assert_nothing_raised() do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest :autogen => :to_default_file

      Mxx_ru::Cpp::Toolsets::Vc8::manifest :autogen => 'some_file'
    end

    assert_raise( Mxx_ru::Invalid_value_ex ) do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest :autogen => :to_def_file
    end

    assert_raise( Mxx_ru::Invalid_value_ex ) do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest :autogen => 1
    end
  end

  def test_keep_manifest_file
    assert_nothing_raised() do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest \
        :autogen => :to_default_file,
        :mt => { :keep_manifest_file => true }

      Mxx_ru::Cpp::Toolsets::Vc8::manifest \
        :autogen => :to_default_file,
        :mt => { :keep_manifest_file => false }
    end

    assert_raise( Mxx_ru::Invalid_value_ex ) do
      Mxx_ru::Cpp::Toolsets::Vc8::manifest \
        :autogen => :to_default_file,
        :mt => { :keep_manifest_file => 1 }
    end
  end
end


